package com.io.yutian.aulib.nbt;

import com.google.common.collect.Maps;

import java.util.*;

public class NBTCompound implements INBT {

    public static byte TYPE_ID = 10;

    private Map<String, INBT> nbtMap = new HashMap<>();

    public NBTCompound() {
    }

    public NBTCompound(Map<String, INBT> nbtMap) {
        this.nbtMap = nbtMap;
    }

    public Set<String> keySet() {
        return nbtMap.keySet();
    }

    public Map<String, INBT> getMap() {
        return nbtMap;
    }

    public void remove(String key) {
        nbtMap.remove(key);
    }

    public boolean hasKey(String key) {
        return this.nbtMap.containsKey(key);
    }

    public boolean hasKey(String key, byte id) {
        return this.nbtMap.containsKey(key) && this.nbtMap.get(key).getTypeId() == id;
    }

    public boolean hasKey(String key, int id) {
        return this.nbtMap.containsKey(key) && this.nbtMap.get(key).getTypeId() == id;
    }

    public NBTList getList(String key) {
        INBT list = this.nbtMap.get(key);
        if (list == null || list.getTypeId() != 9) {
            return null;
        }
        return (NBTList) list;
    }

    public NBTCompound getCompound(String key) {
        INBT comound = this.nbtMap.get(key);
        if (comound == null || comound.getTypeId() != 10) {
            return null;
        }
        return (NBTCompound) comound;
    }

    public INBT get(String key) {
        return this.nbtMap.get(key);
    }

    public int getInt(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 3) {
            return 0;
        }
        return ((NBTInt) nbt).getInt();
    }

    public String getString(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 8) {
            return null;
        }
        return ((NBTString) nbt).getString();
    }

    public double getDouble(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 6) {
            return 0.0;
        }
        return ((NBTDouble) nbt).getDouble();
    }

    public float getFloat(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 5) {
            return 0.0F;
        }
        return ((NBTFloat) nbt).getFloat();
    }

    public short getShort(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 2) {
            return 0;
        }
        return ((NBTShort) nbt).getShort();
    }

    public long getLong(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 4) {
            return 0;
        }
        return ((NBTLong) nbt).getLong();
    }

    public byte getByte(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 1) {
            return 0;
        }
        return ((NBTByte) nbt).getByte();
    }

    public NBTByteArray getByteArray(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 7) {
            return null;
        }
        return (NBTByteArray) nbt;
    }

    public NBTIntArray getIntArray(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 11) {
            return null;
        }
        return (NBTIntArray) nbt;
    }

    public NBTLongArray getLongArray(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 12) {
            return null;
        }
        return (NBTLongArray) nbt;
    }

    public boolean getBoolean(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 1) {
            return false;
        }
        return ((NBTByte) nbt).getByte() != 0;
    }

    public UUID getUUID(String key) {
        INBT nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 11) {
            return null;
        }
        return a(((NBTIntArray) nbt).getIntArray());
    }

    public void put(String key, INBT nbt) {
        this.nbtMap.put(key, nbt);
    }

    public void putString(String key, String value) {
        put(key, new NBTString(value));
    }

    public void putInt(String key, int value) {
        put(key, new NBTInt(value));
    }

    public void putShort(String key, short value) {
        put(key, new NBTShort(value));
    }

    public void putDouble(String key, double value) {
        put(key, new NBTDouble(value));
    }

    public void putLong(String key, long value) {
        put(key, new NBTLong(value));
    }

    public void putFloat(String key, float value) {
        put(key, new NBTFloat(value));
    }

    public void putByte(String key, byte value) {
        put(key, new NBTByte(value));
    }

    public void putBoolean(String key, boolean value) {
        put(key, NBTByte.a(value));
    }

    public void putUUID(String key, UUID uuid) {
        put(key, new NBTIntArray(a(uuid)));
    }

    private static UUID a(int[] var0) {
        return new UUID((long)var0[0] << 32 | (long)var0[1] & 4294967295L, (long)var0[2] << 32 | (long)var0[3] & 4294967295L);
    }

    private static int[] a(UUID var0) {
        long var1 = var0.getMostSignificantBits();
        long var3 = var0.getLeastSignificantBits();
        return a(var1, var3);
    }

    private static int[] a(long var0, long var2) {
        return new int[]{(int)(var0 >> 32), (int)var0, (int)(var2 >> 32), (int)var2};
    }

    @Override
    public Object getValue() {
        return nbtMap;
    }

    @Override
    public byte getTypeId() {
        return TYPE_ID;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        NBTCompound that = (NBTCompound) o;
        Map<String, INBT> nbtMap1 = this.nbtMap;
        Map<String, INBT> nbtMap2 = that.nbtMap;
        if (nbtMap1.size() != nbtMap2.size()) {
            return false;
        }
        for (Map.Entry<String, INBT> entry : nbtMap1.entrySet()) {
            String key = entry.getKey();
            INBT value = entry.getValue();
            if (!nbtMap2.containsKey(key)) {
                return false;
            }
            INBT nbt2 = nbtMap2.get(key);
            Object object1 = value.getValue();
            Object object2 = nbt2.getValue();
            if (value == null && nbt2 != null) {
                return false;
            } else if (object1 instanceof Number && object2 instanceof Number) {
                if (Double.compare(((Number) object1).doubleValue(), ((Number) object2).doubleValue()) != 0) {
                    return false;
                }
            } else if (!object1.equals(object2)) {
                return false;
            }
        }
        return true;
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(nbtMap);
    }

    @Override
    public String toString() {
        return "NBTCompound{" +
                "nbtMap=" + nbtMap +
                '}';
    }

}
