package com.io.yutian.aulib.sql.builder.impl;

import com.io.yutian.aulib.sql.action.query.PreparedQueryActionImpl;
import com.io.yutian.aulib.sql.api.action.query.PreparedQueryAction;
import com.io.yutian.aulib.sql.api.builder.TableQueryBuilder;
import com.io.yutian.aulib.sql.manager.SQLManagerImpl;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

import static com.io.yutian.aulib.sql.api.SQLBuilder.withBackQuote;

public class TableQueryBuilderImpl
        extends AbstractConditionalBuilder<TableQueryBuilder, PreparedQueryAction>
        implements TableQueryBuilder {


    protected final @NotNull String tableName;

    String[] columns;

    @Nullable String orderBy;

    int[] pageLimit;

    public TableQueryBuilderImpl(@NotNull SQLManagerImpl manager, @NotNull String tableName) {
        super(manager);
        this.tableName = tableName;
    }

    @Override
    public PreparedQueryActionImpl build() {
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("SELECT").append(" ");
        if (columns == null || columns.length < 1) {
            sqlBuilder.append("*");
        } else {
            for (int i = 0; i < columns.length; i++) {
                String name = columns[i];
                sqlBuilder.append(withBackQuote(name));
                if (i != columns.length - 1) {
                    sqlBuilder.append(",");
                }
            }
        }

        sqlBuilder.append(" ").append("FROM").append(" ").append(withBackQuote(tableName));

        if (hasConditions()) sqlBuilder.append(" ").append(buildConditionSQL());

        if (orderBy != null) sqlBuilder.append(" ").append(orderBy);

        if (pageLimit != null && pageLimit.length == 2) {
            sqlBuilder.append(" LIMIT ").append(pageLimit[0]).append(",").append(pageLimit[1]);
        } else if (limit > 0) {
            sqlBuilder.append(" ").append(buildLimitSQL());
        }


        return new PreparedQueryActionImpl(getManager(), sqlBuilder.toString())
                .setParams(hasConditionParams() ? getConditionParams() : null);
    }

    @Override
    public @NotNull String getTableName() {
        return tableName;
    }

    @Override
    public TableQueryBuilderImpl selectColumns(@NotNull String... columnNames) {
        Objects.requireNonNull(columnNames, "columnNames could not be null");
        this.columns = columnNames;
        return this;
    }

    @Override
    public TableQueryBuilder orderBy(@NotNull String columnName, boolean asc) {
        Objects.requireNonNull(columnName, "columnName could not be null");
        this.orderBy = "ORDER BY " + withBackQuote(columnName) + " " + (asc ? "ASC" : "DESC");
        return this;
    }

    @Override
    public TableQueryBuilder setPageLimit(int start, int end) {
        this.pageLimit = new int[]{start, end};
        return this;
    }

    @Override
    protected TableQueryBuilderImpl getThis() {
        return this;
    }
}
